import os
from json import dumps

import click

from .loader import (
    load_default,
    load_file,
    load_system,
    load_api,
)


@click.command()
@click.version_option()
@click.option(
    '--api',
    is_flag=False,
    default='https://cli.mindswire.com/env',
    help='Endpoint for API sourced environment varialbes.')
@click.option(
    '-t',
    '--token',
    is_flag=False,
    help='Token for API sourced environment variables before others.')
@click.option(
    '--stage',
    is_flag=False,
    default=None,
    help='Stage for API sourced environment variables.')
@click.option(
    '--no_default',
    is_flag=True,
    help='Do not load from default local file(s).')
@click.option(
    '-s',
    '--system',
    is_flag=True,
    help='Load system environment variables before local ones.')
@click.option(
    '-v',
    '--verbose',
    is_flag=True,
    help='Enables verbose mode.')
@click.option(
    '-f',
    '--format',
    is_flag=False,
    type=click.Choice(['space', 'csv', 'newline', 'json']),
    default='space',
    help='Output format.')
@click.option(  # deprecated, use --format
    '-c',
    '--csv',
    is_flag=True,
    help='Comma instead of space separated KEY=VALUE pairs. DEPRECIATED, use --format csv.')
@click.option(  # deprecated, use --format
    '-j',
    '--json',
    is_flag=True,
    help='Output in JSON string. DEPRECATED, use --format=json.')
@click.argument(
    'files',
    nargs=-1,
    type=click.Path(exists=True, dir_okay=False, resolve_path=True))
def cli(api, token, stage, no_default, system, verbose, format, csv, json, files):
    env_vars = {}

    if system:
        env_vars.update(load_system(verbose))

    if not len(files):
        if not no_default:
            env_vars.update(load_default(verbose))
    else:
        for fname in files:
            env_vars.update(load_file(fname, verbose))

    _token = env_vars.pop('PORTUNUS_TOKEN', os.getenv('PORTUNUS_TOKEN'))
    if not token and _token:
        copy = env_vars.copy()
        env_vars.update(load_api(api, _token, stage, verbose))
        # TODO: let user control the precedence of these env vars
        env_vars.update(copy)

    if env_vars:
        if json or format == 'json':
            return click.echo(dumps(env_vars))

        delimiter = ' '
        if csv or format == 'csv':
            delimiter = ','
        elif format == 'newline':
            delimiter = '\n'

        return click.echo(delimiter.join([
            '{0}={1}'.format(k, v) for k, v in env_vars.items()
        ]))



import os
import sys
import codecs
from setuptools import setup, find_packages
from shutil import rmtree

here = os.path.abspath(os.path.dirname(__file__))
package = "print_env"

with codecs.open(os.path.join(here, "README.md")) as f:
    long_description = "\n" + f.read()

version = {}
with codecs.open(os.path.join(here, package, "version.py")) as f:
    exec(f.read(), version)


def print_run(cmd, err_exit=False):
    print("RUNNING: {}".format(cmd))
    r = os.system(cmd)
    if err_exit and r != 0:
        sys.exit(r)


if sys.argv[-1] == "publish":
    try:
        rmtree(os.path.join(here, "dist"))
    except FileNotFoundError:
        pass

    print_run("{0} setup.py sdist bdist_wheel".format(sys.executable), True)
    print_run("env $(print-env twine.env) twine upload dist/*", True)
    print_run("git tag v{}".format(version["__version__"]))
    print_run("git push --tags")
    sys.exit()

setup(
    name="print-env",
    version=version["__version__"],
    description="CLI to print environment variables from supported sources.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Runzhou Li (Leo)",
    author_email="me@runzhou.li",
    url="https://github.com/woozyking/print-env",
    packages=find_packages(exclude=["tests"]),
    install_requires=[
        "click>=7,<8",
        "python-dotenv==0.19.2",
        "python-gnupg==0.4.8",
        "pyyaml>=5,<6",
        "requests>=2,<3",
    ],
    entry_points={"console_scripts": ["print-env={}.cli:cli".format(package)]},
    package_data={"": ["LICENSE"]},
    license="MIT",
    python_requires=">=3.6",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: PyPy",
    ],
)
